from typing import List, Optional

from sqlalchemy.orm import Session

from app.crud.base import CRUDBase
from app.models.item import Item
from app.schemas.item import ItemCreate, ItemUpdate


class CRUDItem(CRUDBase[Item, ItemCreate, ItemUpdate]):
    """
    CRUD operations for Item model.
    """
    def get_by_sku(self, db: Session, *, sku: str) -> Optional[Item]:
        """
        Get an item by SKU.
        """
        return db.query(Item).filter(Item.sku == sku).first()
    
    def get_by_name(self, db: Session, *, name: str) -> List[Item]:
        """
        Get items by name (can return multiple items).
        """
        return db.query(Item).filter(Item.name.ilike(f"%{name}%")).all()
    
    def get_by_category_id(
        self, db: Session, *, category_id: int, skip: int = 0, limit: int = 100
    ) -> List[Item]:
        """
        Get items by category ID.
        """
        return (
            db.query(Item)
            .filter(Item.category_id == category_id)
            .offset(skip)
            .limit(limit)
            .all()
        )
    
    def get_low_stock_items(
        self, db: Session, *, skip: int = 0, limit: int = 100
    ) -> List[Item]:
        """
        Get items with stock below reorder level.
        """
        return (
            db.query(Item)
            .filter(Item.quantity <= Item.reorder_level)
            .offset(skip)
            .limit(limit)
            .all()
        )
    
    def create_with_owner(
        self, db: Session, *, obj_in: ItemCreate, owner_id: int
    ) -> Item:
        """
        Create a new item with owner.
        """
        obj_in_data = obj_in.model_dump()
        db_obj = Item(**obj_in_data, owner_id=owner_id)
        db.add(db_obj)
        db.commit()
        db.refresh(db_obj)
        return db_obj


# Create a singleton instance
item = CRUDItem(Item)